#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# format_drive_dialog.pyw
# 
# letting the user format a drive attached to a certain VM
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2014 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import base64
import sys

import urllib
import urllib2

from PyQt4 import QtCore
from PyQt4 import QtGui

from ui_FormatDriveDialog import Ui_FormatDriveDialog 
from about_dialog import AboutDialog

# ------------------------------------------------------------
# code


class FormatDriveDialog(QtGui.QDialog):

    """A dialog for letting the user pass on a password/keyfile combo"""

    def __init__(self, ip):

        QtGui.QDialog.__init__(self)

        user_text = """
<b>Attention!</b><br/>
You are going to wipe all data stored<br/>
at device attached to IP <b>%s</b>.<br/>
<br/>
<b>This is irreversible.</b><br/>
<br/>
Please provide an appropriate password or keyfile to proceed:
""" % ip

        # setup the user interface
        self.ui = Ui_FormatDriveDialog()
        self.ui.setupUi(self)
    
        # local members
        self._about_dialog = AboutDialog()
        self._open_file_dialog = QtGui.QFileDialog(self, 'Open Keyfile', QtCore.QDir.homePath(), 'All Files (*.*);;')
        self._open_file_dialog.setAcceptMode(QtGui.QFileDialog.AcceptOpen)
        self._open_file_dialog.setFileMode(QtGui.QFileDialog.ExistingFile)

        # connectors
        self.ui.lblText.setText(user_text)
        self.ui.btnAbout.clicked.connect(self.clicked_about)
        self.ui.btnCancel.clicked.connect(self.reject)
        self.ui.btnOk.clicked.connect(self.clicked_ok)
        self.ui.btnBrowse.clicked.connect(self.clicked_browse)

        # positionate ourself central
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)
 

    def clicked_about(self):

        """About button has been clicked."""
        self._about_dialog.show()


    def clicked_browse(self):

        """Browse button has been clicked."""
        if self._open_file_dialog.exec_() == QtGui.QDialog.Accepted:
            self.ui.edtKeyfile.setText(self._open_file_dialog.selectedFiles()[0])


    def clicked_ok(self):
        
        """Ok button has been clicked."""

        init_data = dict()
        
        # pick the password
        init_data['password'] = str(self.ui.edtPassword.text())
        if len(init_data['password']) == 0:
            QtGui.QMessageBox.critical(self, 'Format error', 'Please specify a password.')
            return

        # read the content of the keyfile
        keyfile_content = ''
        try:
            if len(self.ui.edtKeyfile.text()) > 0:
                keyfile = open(self.ui.edtKeyfile.text(), 'r')
                keyfile_content = keyfile.read()
        except Exception as e:
            sys.stderr.write('failed to read keyfile content:\n' + str(e) + '\n')
            QtGui.QMessageBox.critical(self, 'Failed to read keyfile', str(e))
            return

        # turn into Base64
        if len(keyfile_content) > 0:
            keyfile_content_base64 = base64.b64encode(keyfile_content)
            init_data['keyfile'] = keyfile_content_base64

        res = ""
        try:
            req_data = urllib.urlencode(init_data)
            req = urllib2.Request('http://' + ip + ':58081/init', req_data)
            res = urllib2.urlopen(req)
        except:
            print('EXCEPTION ' + res)
            pass

        self.accept()


    def set_user_text(self, user_text):

        """Set a text to explain which password we need."""
        self.ui.lblText.setText(user_text)


if __name__ == "__main__":
    
    print(sys.argv)
    
    ip = None
    try:
        ip = sys.argv[-1:][0]
    except:
        pass
    
    a = QtGui.QApplication(sys.argv)
    d = FormatDriveDialog(ip)
    d.show()
    sys.exit(a.exec_())     

